/* Asynchronous write, 64bit offset version.
   Copyright (C) 1997, 1999, 2005 Free Software Foundation, Inc.
   This file is part of the GNU C Library.
   Contributed by Ulrich Drepper <drepper@cygnus.com>, 1997.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.  */

#include <aio.h>

#include <aio_misc.h>


int
aio_write64 (aiocbp)
     struct aiocb64 *aiocbp;
{
  return (__aio_enqueue_request ((aiocb_union *) aiocbp, LIO_WRITE64) == NULL
	  ? -1 : 0);
}
