/* Macros to support TLS testing in times of missing compiler support.
   Stub version.

   These macros should yield int * expressions for the TLS symbol X
   accessed using the various TLS access models.  Macros for some machines
   are defined in elf/tls-macros.h, but ports can instead provide this file.

#define TLS_LE(x)
#define TLS_IE(x)
#define TLS_LD(x)
#define TLS_GD(x)
*/
