/* BSD `setjmp' entry point to `sigsetjmp (..., 1)'.  MIPS64 version.
   Copyright (C) 1996, 1997, 2002, 2003, 2004 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library.  If not, see
   <http://www.gnu.org/licenses/>.  */

/* This just does a tail-call to `__sigsetjmp (ARG, 1)'.
   We cannot do it in C because it must be a tail-call, so frame-unwinding
   in setjmp doesn't clobber the state restored by longjmp.  */

#include <sgidefs.h>
#include <sysdep.h>
#include <sys/asm.h>

#ifdef PIC
	.option pic2
#endif
ENTRY (setjmp)
#ifdef __PIC__
	SETUP_GP
#endif
	SETUP_GP64 (v0, C_SYMBOL_NAME (setjmp))
	PTR_LA t9, C_SYMBOL_NAME (__sigsetjmp)
#if _MIPS_SIM == _ABIO32
	nop
#endif	
	RESTORE_GP64
	dli a1, 1		/* Pass a second argument of one.  */
#ifdef __PIC__
	jr	t9
#else
	j	C_SYMBOL_NAME (__sigsetjmp)
#endif
	.end	setjmp
